/**
 * An command interface to send key commands to the unit. 
 * Initiates Ajax request and maintains a discarding Queue.
 */
Ext.define('FP.utils.CmdInterface', {


    config: {
        requestQSize: 1
    },
    
    /**
     * @cfg {Number} requestQSize The size of the Request Queue
     */
    constructor: function(config) {
        this.initConfig(config);
        this.requestQueue = [];
        this.currentRequest = null;
    },
    /**
     * Send command to unit
     * @param {String} keyCodeIn The code represented in the unit for a key press
     *  example: !1
     * @param {Function} cbIn callback to handle response 
     */
    sendCmd: function(keyCodeIn, cbIn) {
        var newRequest = {
            cbfn: cbIn,
            keyCode: keyCodeIn
        };
        if(this.requestQueue.length < this.getRequestQSize()) {
            this.requestQueue.push(newRequest)
            this._doRequest();
        } else if(keyCodeIn !== '!R') {
            
            this.requestQueue[this.requestQueue.length - 1] = newRequest;
        }
    },
    /**
     * @private do the ajax request
     */
    _doRequest: function() {
        if(this.requestQueue.length && (this.currentRequest === null)) {
            this.currentRequest = this.requestQueue.pop();
            
            Ext.Ajax.request({
                url: '/cgi-bin/interface',
                success: this.successCB,
                failure: this.failureCB,
                scope: this,
                method: 'POST',
                jsonData: this.currentRequest.keyCode
            });
            
        }
    },
    /**
     * @private handle success
     */
    successCB: function(res) {
        this.currentRequest.cbfn(res)
        this.currentRequest = null;
        this._doRequest();
    },
    /**
     * @private handle failure
     */
    failureCB: function() {
        this.currentRequest = null;
        this._doRequest();
    }
});
