/**
 * A way around the same origin policy. This class listens to messages from a parent window
 * by registering to window.onMessage events
 */
Ext.define('FP.utils.IframeSlave', {
    extend: 'FP.utils.Observable',
    config: {
        originURI: null
    },
    /**
     * call once ready to receive events
     * @param {Ext.app.Controller} controller A controller with support for sendKeyCmd
     */
    loadController: function(controller) {
        BLogger.log(controller);
        BLogger.log('Loaded Iframe master @ ' + this.getOriginURI());
        controller.on('ledchange', this.inputChange, this);
        Ext.get(window).addListener('message', function(msg) {
            var brsrEvt = msg.browserEvent;
            //console.log("got message  " +  brsrEvt.data);
            controller.sendKeyCmd(brsrEvt.data);
        });
    },
    /**
     * Method to call when state of the front panel has changed.
     * Notifies the parent window with state
     * @param {Object} inputObj. Object containing state of button LEDs, and VFD
     *  display
     */
    inputChange: function(inputObj) {
        //console.log("posting message to parent " + inputObj);
        window.parent.postMessage(Ext.JSON.encode(inputObj), this.getOriginURI());
    }
    
});

