
Ext.define('FP.view.FrontPanel', {
    requires: [
        'FP.view.PanelKey'
    ],
    
    alias: 'widget.frontpanel',
    extend: 'Ext.Component',
    id: 'main',
//    width: 800,
//    height: 600,
    bgRatio: 1.793,
    inputKeySpacing: 76.5,
    textX: 150,
    textY: 52,
    navigationKeys: {
        x: 491,
        y: 48,
        spaceX: 20 + 58, /* Space + width of key */
        spaceY: 24 + 56,
        defaults: {
            
        },
        items: [
            {
                name: 'key_up',
                imgSrc: '/fp/resources/images/key_up.png',
                locationScale: {
                    x: 0,
                    y: 0
                }
            }, {
                name: 'key_down',
                imgSrc: '/fp/resources/images/key_down.png',
                locationScale: {
                    x: 0, 
                    y: 1
                }
            }, {
                name: 'key_upfast',
                imgSrc: '/fp/resources/images/key_fastup.png',
                locationScale: {
                    x: 1, 
                    y: 0
                }
            }, {
                name: 'key_downfast',
                imgSrc: '/fp/resources/images/key_fastdown.png',
                locationScale: {
                    x: 1, 
                    y: 1
                }
            }, {
                name: 'key_sel',
                imgSrc: '/fp/resources/images/key_sel.png',
                locationScale: {
                    x: 2.5, 
                    y: 0
                }
            }, {
                name: 'key_esc',
                imgSrc: '/fp/resources/images/key_esc.png',
                locationScale: {
                    x: 2.5, 
                    y: 1
                }
            }
        ]
    },
    effectsKeys: {
        x: 256,
        y: 362,
        spaceX: 58*2,
        spaceY: 0,
        defaults: {
            
        },
        items: [
            {
                name: 'key_frz',
                text: 'FRZ',
                locationScale: {
                    x: 0,
                    y: 0
                }
            },
            {
                name: 'key_blk',
                text: 'BLK',
                locationScale: {
                    x: 1,
                    y: 0
                }
            },
            {
                name: 'key_take',
                highlightFill: 'red',
                text: 'TAKE',
                locationScale: {
                    x: 2,
                    y: 0
                }
            },
            
        ]
        
    },
    additionalTextOffset: 30,
    constructor: function() {
        this.addEvents(['keypressed']);
        this.callParent(arguments);
    },
    afterRender: function() {
        BLogger.log('Front Panel loaded');
        
        this.keys = [];
        this.keysByName = {};
        this.callParent(arguments);
        
        var stage = new Kinetic.Stage({
                container: this.el,
                width: 800,
                height: 800/this.bgRatio
            }),
            img = new Image(),
            me = this
        
        img.src = '/fp/resources/images/controller-bg.png';

        this.layer = new Kinetic.Layer();
        
        this.bgLayer = new Kinetic.Layer();
        
        
        stage.add(this.layer);
        stage.add(this.bgLayer);
        this.layer.setZIndex(0);
        this.bgLayer.setZIndex(0);
        
        
        //create 10 input keys
        for(var i = 0; i < 10; i++) {
            var text;
            if(i<8) {
                text = '' + (i+1);
            } else if(i==8) {
                text = 'SDI'
            } else {
                text = 'LOGO'
            }
            this.keys.push(Ext.create('FP.view.PanelKey', {
                name: 'key_'+ (i + 1),
                stage: this.layer, 
                location: {
                    x: 25 + this.inputKeySpacing*i,
                    y: 258
                }, 
                text: text,
                listeners: {
                    'pressed': function(key) {
                        me._keyPressed(key);
                    }
                }
            }));
        }
        
        //create and layout other keys
        this.layoutKeys(this.navigationKeys);
        this.layoutKeys(this.effectsKeys);
        

        img.onload = function() {
            
            me.bgLayer.add(new Kinetic.Image({image: img, width: 800, height: 800/(me.bgRatio)}));
            
            me.bgLayer.draw();
            me.layer.draw();
        };
        
        var textConfig = {
            x: this.textX,
            y: this.textY,
            text: "Loading...",
            fontSize: 16,
            fontFamily: "Monospace",
            fontStyle: 'bold',
            textFill: '#00FFFF',
            padding: 5
        }
        this.text = []
        for(var idx = 0; idx < 4; idx++) {
            var textItem = new Kinetic.Text(
                Ext.apply(textConfig, 
                    {
                        y: this.textY + this.additionalTextOffset*idx
                    }
                )
            );
            this.layer.add(textItem);
            this.text.push(textItem);
        }
        
//        var container = Ext.getCmp('center-container');
        var resize = function() {
            var newHeight = window.innerHeight - 10,
                newWidth;
            if(newHeight > 800/me.bgRatio) {
                BLogger.log('normal');
                return;
            }
            newWidth = newHeight * me.bgRatio;
//            if(newWidth > (c.getWidth()-100)) {
//                console.error('too wide');
//                newWidth = c.getWidth() - 180;
//                newHeight = newWidth/me.bgRatio;
//            }
            BLogger.log(newWidth, newHeight)
            stage.setSize(newWidth, newHeight);
            stage.setScale(newWidth/800.)
            stage.draw();
        }
        window.addEventListener('resize', function() {
            resize();
            
        })
        Ext.onReady(function() {
            resize();
        });
        this.runScheduler();
        
    },
    layoutKeys: function(config){
        for(var idx = 0; idx < config.items.length; idx+=1) {
            var currPrototype = config.items[idx];
            var opts = Ext.apply(config.defaults, currPrototype);
            Ext.apply(opts, 
                {
                    stage: this.layer,
                    location: {
                        x: config.x + (config.spaceX)*currPrototype.locationScale.x,
                        y: config.y + (config.spaceY)*currPrototype.locationScale.y
                    },
                    listeners: {
                        'pressed': this._keyPressed,
                        scope: this
                    }
                }
            );
            var panelKey = new Ext.create('FP.view.PanelKey', opts);
            this.keys.push(panelKey);
            this.keysByName[opts.name] = panelKey;
        }
    },
    _keyPressed: function(key) {
        this.fireEvent('keypressed', key.name);
    },
    setVFDText: function(convertedText) {
        for(var idx = 0; idx < this.text.length; idx++) {
            if(convertedText[idx] !== undefined) {
                this.text[idx].setText(convertedText[idx]);
            }
        }
        this.layer.draw();
    },
   
    setLedState: function(ledStateObj) {
        var method = {
            0: 'lightOff',
            1: 'lightOn',
            2: 'lightFlash'
        }
        this.keys.forEach(function(key) {
            var state = ledStateObj[key.name];
            if(state !== undefined) {
//                BLogger.log(key.name, method[state]);
                key[method[state]].call(key);
            }
        });
    },
    
    runScheduler: function() {
            var me = this,
            cb = null,
            schedule = null;
            
        schedule = function() {
            window.setTimeout(cb, 500);
        }
        cb = function() {
//            BLogger.log('callback');
            for(var keyidx = 0; keyidx < me.keys.length; keyidx+=1) {
                if(typeof me.keys[keyidx].onTick === 'function') {
                    me.keys[keyidx].onTick();
                }
            }
            me.layer.draw();
            schedule();
        }
        schedule();
    }

});
