/**
 * 
 *
 * 
 */

Ext.define('Muc.controller.Muc', {
    extend: 'Ext.app.Controller',
    requires: ['Ext.Ajax', 'Muc.manager.IFrameManager', 'Muc.view.Growl'],
    refs: [
        {
            ref: 'controlGrid',
            selector: 'grid'
        }, {
            ref: 'unitForm',
            selector: 'UnitEdit'
        }, {
            ref: 'selectView',
            selector: 'selectview'
        }, {
            ref: 'discoveryView',
            selector: 'discovery'
        }
    ],
    stores: ['UnitS', 'DiscoveryS'],
    models: ['UnitM'],
    init: function() {
        window.cont = this;
        this.iframes = {};
        this.iframeEvts = {};
        var me = this,
            store = this.getUnitSStore();
        //runs before the state of the world
        this.control({
            'unitsgrid': {
                selectionchange: this.gridRowSelected
            },
            'unitsgrid button[action=saveunit]':  {
                'click': function() {
                    var proxy = store.getProxy();
                    proxy.saveOnUnit(store, function(){
                        me.growl.msg("System", "Saved Settings to " + window.location.host);
                    }, function() {
                        me.growl.msg("System", "Save Failure");
                    });
                }
            },
            'unitsgrid button[action=remove]': {
                click: function() {
                    var grid = me.getControlGrid(),
                        selModel = grid.getSelectionModel();
                    store.remove(selModel.getSelection());
                    Ext.getCmp('editcard').getLayout().setActiveItem(0);
                }
            },
            'UnitEdit button[action=update]': {
                click: this.updateUnitInfo
            },
            'unitsgrid button[action=add]': {
                click: this.addUnit
            },
            'selectview': {
                navigate: function(direction) {
                   
                }
            },
            'discovery button[action=discover]': {
                click: this.discover
            },
            'discovery button[action=add]': {
                click: this.addFromDiscovery
            },
            '#inputbus button': {
                click: function(btn) {
                    me.handleInputButtonPress(btn.value);
                }
            }
        });

        
        
        this.iframeManager = Ext.create('Muc.manager.IFrameManager', 
            {
                store: store,
                controller: this
            }
        );
        this.growl = Ext.create('Muc.view.Growl');
            
        this.getDiscoverySStore().on('load', function(store) {
            var count,
                unitOrUnits;
            me.getDiscoveryView().setLoading(false);
            count = store.getCount();
            if(count === 1) {
                unitOrUnits = " Unit";
            } else {
                unitOrUnits = " Units";
            }
            me.growl.msg('Discovery', 'Found ' + count + unitOrUnits);
        });
        this.createKeyBoardShortCuts();
    },
    /**
     *  handler for grid selection change. Load the proper record in the edit box. 
     */
    gridRowSelected: function(selModel, record) {
        var tabPanel;
        if(record.length > 0) {
            
            this.getUnitForm().loadRecord(record[0]);
            Ext.getCmp('editcard').getLayout().setActiveItem(1);
        } else {
            Ext.getCmp('editcard').getLayout().setActiveItem(0);
        }
        //switch tab to edit view
        tabPanel = this.getUnitForm().up('tabpanel');
        tabPanel.layout.setActiveItem(0);
    },
    /**
     *
     * Apply the edited info from the form panel to the store. 
     */
    updateUnitInfo: function(button) {
        var form = button.up('UnitEdit'),
            record = form.getRecord(),
            values = form.getValues();
        
        
        record.set(values);
    },
    addUnit: function(button) {
        var record = this.getUnitMModel().create(),
            selectionModel = this.getControlGrid().getSelectionModel();
        
        this.getUnitSStore().add(record);
        selectionModel.select(record, false);
        
    },
    addFromDiscovery: function(button){
        var grid = button.up('discovery'),
            selectModel = grid.getSelectionModel(),
            selectedItems = selectModel.getSelection(),
            idx;
            
        for(idx = 0; idx < selectedItems.length; idx+=1) {
            this.getUnitSStore().add(selectedItems[idx].copy());
        }
        this.getDiscoverySStore().remove(selectedItems);
    },
    onLaunch: function() {
        var me = this;
        this.getUnitSStore().on('load', function() {
            window.setTimeout(function() {
                //XXX: for reason, view is not styled correctly if not delayed
                //on initial load
                me.getSelectView().hookDivEvents();
            }, 1000)
        });
    },
    /**
     * Perform UDP discovery. This is done through the store.;
     */
    discover: function () {
        var view = this.getDiscoveryView();
        this.getDiscoverySStore().load();
        view.setLoading(true);
        
        //make sure we timeout if unit is disconnected
        window.setTimeout(function(){
            view.setLoading(false);
        }, 10000);
    },
    updateStateFromEvt: function(evt, ip) {
        var prop = null,
            keyData = Ext.JSON.decode(evt.data),
            keyNumber,
            keyState,
            preview = 0,
            input = 0;
        for(prop in keyData) {
            if(keyData.hasOwnProperty(prop)) {
                keyNumber = prop.replace('key_', '');
                keyState = keyData[prop];
                if(keyState === 1) {
                    input = keyNumber;
                    break;
                }
            }
        }
        return this.updateState(ip, input, keyData.key_frz);
    },
    updateState: function(ip, input, freeze) {
        var record = this.getUnitSStore().findRecord('ip-address', ip),
            status = false;

        if(record == null) {
            Ext.Logger.error('Could not update program/preview, unable to find record for IP: ' + ip);
        } else {
            record.set('connection', "Connected");
            record.set('input', input);
            record.set('freeze', freeze);
            status = true;
        }
        return status;
    },
    
    updateConnectionState: function(ip, state) {
        var record = this.getUnitSStore().findRecord('ip-address', ip),
            status = false;
        if(record == null) {
            Ext.Logger.error('Could not update program/preview, unable to find record for IP: ' + ip);
        } else {
            record.set('connection', state);
            status = true;
        }
        return status;
    },
    createKeyBoardShortCuts: function() {
        var me = this,
            key,
            keyCodes = {
                '1': '!1',
                '2': '!2',
                '3': '!3',
                '4': '!4',
                '5': '!5',
                '6': '!6',
                '7': '!7',
                '8': '!8',
                '9': '!I', //SDI
                '0': '!L', //LOGO
                't': '!T',
                'T': '!T',
                's': '!I',
                'S': '!I',
                'l': '!L',
                'L': '!L'

            };
        
        
        for(key in keyCodes) {
            if(keyCodes.hasOwnProperty(key)) {
                (function(keyIn, codeIn) {
                    Mousetrap.bind(keyIn, function() {
                        me.handleInputButtonPress(codeIn);
                    })
                })(key, keyCodes[key]);
            }
        }
    },
    
    /**
     * Send a code (represents a key press on the front panel of the device) 
     *  to the IFrame 
     *@param keyCode 
     * 
     */
    handleInputButtonPress: function(keyCode) {
        var selectedItems = this.getUnitSStore().queryBy(function(record){return record.get('selected')}),
            idx,
            currIp;
                        
        for(idx=0; idx < selectedItems.length; idx+=1) {
            currIp = selectedItems.getAt(idx).get('ip-address');
            if(currIp) {
                this.iframeManager.postMessage(currIp, keyCode);
            }
        }
    }

});
