/**
 * The Select View widget. This is the side scrollable widget that allows the 
 *  user to select the units for sending commands, although it doesn't handle the
 *  actual sending.  
 */
Ext.define('Muc.view.SelectView', {
    alias: 'widget.selectview',
    extend: 'Ext.view.View',
    tpl: [
        '<div class="select-view-container">',
            '<div class="nav_key left_key nav-larrow_inactive">',
            '</div>',
            
            '<div class="outer-select">',
                '<div class="inner-select">',
                    '<tpl for=".">',
                        '<div class="select-item',
                                "<tpl if='selected'>",
                                    ' item-selected',
                                "</tpl>",
                            '">',
                            '<div class="select-unit">',
                                '<fieldset>',
                                    '<legend>',
                                    ' Name ',
                                    '</legend>',
                                    '<span>',
                                    '{name}',
                                    '</span>',
                                '</fieldset>',
                                '<fieldset class="align-left">',
                                    '<legend>',
                                    ' IN ',
                                    '</legend>',
                                    '<span>',
                                    '{input}',
                                    '</span>',
                                '</fieldset>',
                                '<fieldset class="align-left">',
                                    '<legend>',
                                    ' FRZ ',
                                    '</legend>',
                                    '<span>',
                                    '{freeze}',
                                    '</span>',
                                '</fieldset>',
                                '<fieldset>',
                                    '<legend>',
                                    ' Status ',
                                    '</legend>',
                                    '<span class="state-{connection}">',
                                    '{connection}',
                                    '</span>',
                                '</fieldset>',
                            '</div>',
                        '</div>',
                    '</tpl>',
                '</div>',
            '</div>',
            
            '<div class="nav_key right_key nav-rarrow_inactive">',
            '</div>',
        '</div>'
    ],
    config: {
        /**
         * @cfg {Number} totalItemWidth optional width of each unit
         */
        totalItemWidth: 210,
        /**
         * @cfg {Number} pageSize How many units to support per "page"
         */
        pageSize: 4
    },
    store: 'UnitS',
    itemSelector: 'div.select-item',
    
    /**
     * @private add listeners set select mode, etc
     */
    
    initComponent: function() {
        this.initialLoad = true;
        this.on('itemclick', function(me, record) {
            var selected = record.get('selected');
            record.set('selected', !selected);
        });
        this.addListener('itemadd', function() {
            //console.log('item added ' + this.store.getCount());
            this.hookDivEvents();
        }, this);
        this.addListener('itemremove', function() {
//            console.log('item removed ' + this.store.getCount());
            this.hookDivEvents();
        }, this);
        Ext.apply(this, {
            selModel: {
                mode: 'MULTI',
                deselectOnContainerClick: false
            },
            contentPosition: 0
        });
        this.callParent(arguments);
    },
    
    /**
     * @private register for div events for navigation
     */
    hookDivEvents: function() {
        console.log("HOOKING DIV EVENTS");
        if(this.rightKey === this.el.select('.right_key')) {
            Ext.Logger.error('damn it');
        }
        if(this.rightKey) {
            this.rightKey.un('click', this.onRightKeyClick, this)
        }
        if(this.leftKey) {
            this.leftKey.un('click', this.onleftKeyClick, this)
        }
        this.rightKey = this.el.select('.right_key');
        this.leftKey = this.el.select('.left_key');
        this.rightKey.on('click', this.onRightKeyClick, this);
        this.leftKey.on('click', this.onLeftKeyClick, this);
        
        this.iselect= this.el.select('.inner-select');
        this.setNavigationStyles();
        if(this.initialLoad) {
            this.scrollToIdx(0);
            this.initialLoad = false;
        }
    },
    
    /**
     *scroll to the index of a unit. Always normalizes to the first unit on that
     * page according to {@link #pageSize}
     *@param {Number} idx Index of the unit. 
     */
    scrollToIdx: function(idx) {
        idx = this.getClosestStartIdx(idx);
        
        var shift = -this.totalItemWidth*idx+ 'px';
        this.iselect.setLeft(shift, true);
        this.contentPosition = idx;
        this.setNavigationStyles();
    },
    
    /**
     * Scroll to a page by {@link #pageSize}
     */
    scrollToPage: function(page) {
        this.scrollToIdx(page*this.pageSize);
    },

    /**
     * get the current position in unit index
     * @return {Number} unit index
     */
    getScrollIdx: function() {
        return this.contentPosition;
    },
    
    /**
     * get the current position in page index
     * @return {Number} page index
     */
    getCurrPageIdx: function() {
        return this.contentPosition/this.pageSize;
    },
    
    /**
     * @private get the idx of first unit on the page
     * @param {Number} idx index of the page
     */
    getClosestStartIdx: function(idx) {
        idx = Math.min(Math.max(0, idx), (this.store.getCount() - 1));
        return parseInt(idx/this.pageSize)*this.pageSize;
    },
    
    /**
     * get the total page count or optionally to toIdx.
     * @param {Number} toIdx An Optional index to get the count to defaults to last 
     */
    getTotalPageCount: function(toIdx) {
        var additionalPage = null;
        if(typeof toIdx === 'undefined') {
            //total page count
            toIdx = this.store.getCount();
        }
        additionalPage = (toIdx % this.pageSize > 0) ? 1: 0;
        return parseInt(toIdx / this.pageSize) + additionalPage;
    },
    
    /**
     * @private Set the style of the navigation arrows. Should be redone when
     * items are added/removed
     */
    setNavigationStyles: function() {
        console.log('total page count:', this.getTotalPageCount());
        var currPageIdx = this.getCurrPageIdx();
        if(currPageIdx > 0) {
            this.leftKey.removeCls('nav-larrow_inactive');
            this.leftKey.addCls('nav-larrow_active');
        } else {
            this.leftKey.removeCls('nav-larrow_active');
            this.leftKey.addCls('nav-larrow_inactive');
        }
        
        if(currPageIdx < (this.getTotalPageCount() - 1)) {
            this.rightKey.removeCls('nav-rarrow_inactive');
            this.rightKey.addCls('nav-rarrow_active');
        } else {
            this.rightKey.removeCls('nav-rarrow_active');
            this.rightKey.addCls('nav-rarrow_inactive');
        }
    },
    
    /**
     * @private
     */
    onLeftKeyClick: function() {
        this.fireEvent('navigate', 'left');
        this.scrollToPage(this.getCurrPageIdx() - 1);
    },
    /**
     * @private
     */
    onRightKeyClick: function() {
        this.fireEvent('navigate', 'right');
        this.scrollToPage(this.getCurrPageIdx() + 1);
    }
});


