
barco.tags = barco.tags || {};

/************************
 * Any XML tag should be placed in barco.tags object
 ************************/
barco.tags.build = function () {
    "use strict";
    this.ActiveConn = 'ActiveConn';
    this.Contr = 'Contr';
    this.Bright = 'Bright';
    this.Gamma = 'Gamma';
    this.Mono = 'Mono';
    this.ColorInvert = 'ColorInvert';
    this.ColorSpace = 'ColorSpace';
    this.AspectRatioType = 'PipAR';
    this.CustomAspectRatio = 'PipCustomAR';
    this.Sharp = 'Sharp';
    this.ContrR = 'ContrR';
    this.ContrG = 'ContrG';
    this.ContrB = 'ContrB';
    this.BrightR = 'BrightR';
    this.BrightG = 'BrightG';
    this.BrightB = 'BrightB';
    this.Hue = 'Hue';
    this.Sat = 'Sat';
    this.RasterBoxMode = 'RasterBoxMode';
    this.DiagMotion = 'DiagMotion';
    this.TestPattern = 'TestPattern';
    this.TestPatternMode = 'TestPatternMode';
    
    this.Name = 'Name';
    this.HSize = 'HSize';
    this.VSize = 'VSize';
    this.HPos = 'HPos';
    this.VPos = 'Vpos'; //not camel case, possible typo
    this.PixelClk = 'PixelClk';
    this.PanZoomUnit = 'PanZoomUnit';
    this.PanZoomUnitValue = 'PanZoomUnitValue';
    this.MaskTop = 'MaskTop';
    this.MaskBottom = 'MaskBottom';
    this.MaskRight = 'MaskRight';
    this.MaskLeft = 'MaskLeft';
    this.InType = 'InType';


    this.Root = 'ImagePRO2';
    this.InputMgr = 'InputMgr';
    this.OutputChan = 'OutputChan';
    this.OutputMgr = 'OutputMgr';
    this.VideoColor = 'VideoColor';
    this.VideoFormat = 'VideoFormat';
    this.OutAOI = 'OutAOI';
    this.InputChan = 'InputChan';
    this.OutputChan = 'OutputChan';
    this.IWIN = 'IWIN';
    this.UncropIWIN = 'UncropIWIN';
    this.PipMask = 'PipMask';
    this.PanZoomPxl = 'PanZoomPxl';
    this.UncroppedOWIN = 'UncroppedOWIN';
    this.InView = 'InView';
    this.UInput = 'UInput';
    this.HActive = 'HActive';
    this.VActive = 'VActive';

    this.DVIConn = 'DVIConn';
    this.HD15Conn = 'HD15Conn';
    this.HDMIInput = 'HDMIInput';
    this.DPInput = 'DPInput';
    this.SDIConn = 'SDIConn';
    this.SDIInput = 'SDIInput';
    
    this.AnalogInput = "AnalogInput";
    this.DVIInput = "DVIInput";

    this.SDIInput1 = this.SDIInput + '[Num="0"]';
    this.SDIInput2 = this.SDIInput + '[Num="1"]';
    
    this.InputChan1 = this.InputChan + ' [Num="0"]';

    this.SystemMgr = 'SystemMgr';
    this.Lock = 'Lock';
    this.OpMode = "OpMode";
    this.AOIRect = 'AOIRect';
    this.FreezeMode = 'FreezeMode';
    this.Mode3D = "Mode3D";
    this.Swap3D = "Swap3D";
    
    //object key cannot be attribute
    var activeConn = this.ActiveConn, maxSize = 2500;
    this.enumVals = {
        //for parsing only
        activeConn: [
            barco.tags.DVIConn,
            barco.tags.HD15Conn,
            barco.tags.HDMIInput,
            barco.tags.DPInput,
            barco.tags.SDIInput1,
            barco.tags.SDIInput2
        ],
        //make up for ImagePRO2's deformed input heirachy.  
        getActiveConnProper: {
            0: barco.tags.DVIConn,
            1: barco.tags.HD15Conn,
            2: barco.tags.HDMIInput,
            3: barco.tags.DPInput,
            4: barco.tags.SDIConn,
            5: barco.tags.SDIConn
        },
        getInputTypeTag: function (conn) {
            var tagattr, 
                inTypeXml, 
                connType = barco.tags.enumVals.getActiveConnProper[conn];
            if (connType === barco.tags.DVIConn) {
                inTypeXml = barco.utils.xmlselect(barco.tags.InType)[0];
                if (inTypeXml) {
                    tagattr = barco.tags.enumVals.DVIInputType[inTypeXml.textContent];
                }
            } else if (connType == barco.tags.SDIConn) {
                tagattr = barco.tags.enumVals.SDIInputType[conn];
            } else if (connType == barco.tags.HD15Conn) {
                tagattr = {tag: barco.tags.AnalogInput, attr: 'Num="0"'};
            }
            return Ext.apply(tagattr, {xml: inTypeXml});
        },
        SDIInputType: {
            4: {tag: barco.tags.SDIInput, attr: 'Num="0"'},
            5: {tag: barco.tags.SDIInput, attr: 'Num="1"'}
        },
        DVIInputType: {
            0: {tag: barco.tags.DVIInput, attr: 'Num="0"'},
            1: {tag: barco.tags.AnalogInput, attr: 'Num="0"'}
        }
    };

    
    this.constraints = {};
    this.constraints[this.HSize] = maxSize;
    this.constraints[this.VSize] = maxSize;
    
    this.buildTemplates();

};

/************************
 * This function constructs a general purpose XML template
 ************************/
barco.tags.buildTemplates = function () {
    "use strict";

    this.templates = {};
    this.templates.spec = 
        [   '<tpl if="root">',
                '<{root}',
                '<tpl if="attr">',
                    ' {attr}>',
                '</tpl>',
                '<tpl if="noattr">',
                    '>',
                '</tpl>',
            '</tpl>',
            '{contents}',
            '<tpl if="root">',
                '</{root}>',
            '</tpl>'].join('');

    this.templates.leafspec = '<{tag}>{value}</{tag}>';
    this.templates.leaf = new Ext.XTemplate(this.templates.leafspec);
    this.templates.leaf.compile();
    
    
    this.templates.root = new Ext.XTemplate(this.templates.spec);
    this.templates.root.compile();
    
    var num0 = 'Num="0"';
    
    this.templates.rootspec = this.templates.root.apply({
        root: this.Root,
        attr: num0,
        contents: this.templates.spec
    });
        
    
};
barco.tags.build();
/* *
 * Given the xml tree/snippet, input manager will select connection child node 
 * based on which input is currently enabled.
 * */
barco.tags.getInputMgr =  function (xml) {
    "use strict";
    var inputxml, activeConn, videoInSubTag, dviintype;
    inputxml = Ext.DomQuery.selectNode(barco.tags.InputMgr, xml);
    activeConn = Ext.DomQuery.selectNode(barco.tags.ActiveConn, inputxml).textContent;
    videoInSubTag = barco.tags.enumVals.activeConn[activeConn];
    if (videoInSubTag != undefined) {
        xml = Ext.DomQuery.selectNode(videoInSubTag, inputxml);
        if(videoInSubTag == barco.tags.DVIConn){
            dviintype = Ext.DomQuery.selectNode(barco.tags.InType, xml).textContent;
            xml = Ext.DomQuery.selectNode(barco.tags.enumVals.DVIInputType[dviintype].tag, xml);
        }
    }
    barco.modelProxy.activeConnection = activeConn;
    return xml;
};


/************************
 * Given a node from XML, this function will construct
 * the complete hierarchy to the root. The template is then cached to
 * barco.tags.templateCache
 ************************/
barco.tags.reverseTemplate = function (leaf, spec) {
    "use strict";
    if (spec === undefined) {
        spec = "{value}";
    }
    if (barco.tags.templateCache === undefined) {
        barco.tags.templateCache = {};
    }
    var attr,
        ta,
        applyTemplate,
        templateBeforeSpec,
        results = [],
        pathID = "",
        getAttr = function (node) {
            if (node.attributes.length > 0) {
                return node.attributes[0].name + "=" + '"' + node.attributes[0].value + '"';
            }
            return null;
        };
    while (leaf !== null) {
        results.push(leaf);
        attr = getAttr(leaf);
        ta = leaf.tagName;
        if (attr !== null) {
            ta += "[" + attr + "]";
        }
        pathID = ta + " " + pathID;
        leaf = leaf.parentElement;
    }
    
    if ((barco.tags.templateCache.hasOwnProperty(pathID)) == false) {
        console.log("New template created: " + pathID);
        results.reverse();
        applyTemplate = function (nodes, spec) {
            var currNode, template, noattr, attrStr;
            if (nodes.length > 0) {
                currNode = nodes[0];
                spec = spec || this.templates.spec;
                template = new Ext.XTemplate(spec);
                if (currNode.attributes.length > 0) {
                    attrStr = getAttr(currNode);
                } else {
                    noattr = true;
                }

                spec = template.apply({
                    root: currNode.tagName, 
                    attr: attrStr,
                    noattr: noattr,
                    contents: this.templates.spec
                });
                return applyTemplate.call(this, nodes.slice(1), spec);
            } else {
                return spec;
            }
        };

        templateBeforeSpec = new Ext.XTemplate(applyTemplate.call(this, results));
        barco.tags.templateCache[pathID] = new Ext.XTemplate(
            templateBeforeSpec.apply({
                noattr: true, 
                contents: spec
            }), 
            {compiled: true}
        );
    }
    return {template: barco.tags.templateCache[pathID], path: pathID};
};

/************************
 * A data stores for sencha's components. They have two levels of filtering.
 * 1) selectXmlSpecific
 * 2) the proxy.reader.record for selecting the root tag afterwards.
 * 
 * The 'apply' function generates XML template from the components tag. This
 * template can then be used to get the actual XML containing the change. 
 * 
 * The model attribute describes what's inside the XML record. 
 * ioType - input, other, output. For keeping track of changes. 
 ************************/
barco.dataStores = {
    defaults: {
        /*proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.OutputMgr
            }
        },*/
        selectXmlSpecific: function (xml) {
            "use strict";
            return xml;
        },
        apply: function (obj, index) {
            "use strict";
            index = index || 0;
            var xml = this.selectXmlSpecific(barco.utils.store.rawxml),
                node = Ext.DomQuery.select(this.proxy.reader.record  + " " + obj.tag, xml)[index],
                template = barco.tags.reverseTemplate(node).template;
            return template.apply(obj);
        },
        ioType: 'other',
        toViewValue: function(tag, val, index){
            "use strict";
            return val;
        },
        fromViewValue: function(tag, val, index){
            "use strict";
            return val;
        }
    },
    stores: [
    {
        barconame: 'OutputMgrStore',
        barcolabel: 'Output Manager',
        model: "Mode3DOnly",
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.OutputMgr
            }
        },
        ioType: 'output'
    }, {
        barconame: 'OutputEffectsStore',
        barcolabel: 'Output Effects',
        model: barco.tags.VideoColor,
        ioType: 'output',
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.OutputChan //+ " " + barco.tags.VideoColor
            }
        }
    }, {
        barconame: 'OutputFormatStore',
        barcolabel: 'Output Format',
        model: barco.tags.VideoFormat,
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.OutputMgr + " " + barco.tags.VideoFormat
            }
        },
        selectXmlSpecific: function (xml) {
            "use strict";
            return Ext.DomQuery.selectNode(barco.tags.OutputMgr, xml);
        },
        ioType: 'output'
    }, {
        barconame: 'InputMainStore',
        barcolabel: 'Input Main',
        model: barco.tags.VideoColor,
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.VideoColor
            }
        },
        selectXmlSpecific: barco.tags.getInputMgr,
        ioType: 'input'
            
    }, {
        barconame: 'TestPatternStore',
        barcolabel: 'Test Pattern',
        model: barco.tags.TestPattern,
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.OutAOI
            }
        }
    }, /*{
        barconame: 'TestPatternModeStore', //need a different store/model for mode
        barcolabel: 'Test Pattern',         // it's a select, need to set value by index
        model: barco.tags.TestPatternMode,
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.OutAOI
            }
        }
    }, */{
       barconame: 'InputManagerTopLevel',
       barcolabel: 'Input Manager',
       model: barco.tags.InputMgr,
       proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.InputMgr
            }
        },
        apply: function(obj) {
            "use strict";
            var tpl = barco.dataStores.defaults.apply.call(this, obj);
            return barco.utils.convertXmlToCmd(tpl);
        }
    }, {
        barconame: 'UInputStore',
        barcolabel: 'Universal Input',
        model: "Mode3DOnly",
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.UInput
            }
        },
        selectXmlSpecific: barco.tags.getInputMgr,
        ioType: 'input'
    }, {
        barconame: 'InputChanIWINStore',
        barcolabel: 'Input Sizing',
        model: barco.tags.IWIN,
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.IWIN
            }
        },
        selectXmlSpecific: barco.tags.getInputMgr,
        ioType: 'input',
        inner: new Ext.XTemplate('<{tag}>{value}</{tag}>'),
        apply: function(obj, index) {
            "use strict";
            //console.debug(index);
            var node = Ext.DomQuery.select("InputChan", this.selectXmlSpecific(barco.utils.store.rawxml))[index],
                template = barco.tags.reverseTemplate(node).template, 
                tag = (obj.tag == 'Vpos' ? 'VPos' : obj.tag), 
                innerObj = {value:obj.value, tag:'IWIN'+tag},
                cmd = template.apply({value:this.inner.apply(innerObj)}),
                ret = barco.utils.convertXmlToCmd(cmd);
            //console.log(ret);
            return ret;
        }
    }, {
        barconame: 'InputFormatStore',
        barcolabel: 'Input Format',
        model: barco.tags.VideoFormat,
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.VideoFormat
            }
        },
        selectXmlSpecific: barco.tags.getInputMgr,
        ioType: 'input'
    }, {
        barconame: 'InputChanStore',
        barcolabel: 'Input Chan',
        model: barco.tags.InputChan,
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.InputChan
            }
        },
        selectXmlSpecific: barco.tags.getInputMgr,
        ioType: 'input'
    },{
        barconame: 'InViewStore',
        barcolabel: "Pan/Zoom",
        model: barco.tags.InView,
        ioType: 'input',
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.PanZoomUnit
            }
        },
        selectXmlSpecific: function (xml) {
            "use strict";
            var activeInput = barco.utils.xmlselectnode(barco.tags.ActiveConn).textContent;
            return Ext.DomQuery.select(barco.tags.InView, xml)[activeInput];
        }
    }, { 
        barconame: 'PanZoomStorePct',
        barcolabel: 'Pan/Zoom',
        model: "Panz",
        ioType: 'input',
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.PanZoomPxl
            }
        },
        selectXmlSpecific: function (xml) {
            "use strict";
            var activeInput = barco.utils.xmlselectnode(barco.tags.ActiveConn).textContent;
            return Ext.DomQuery.select(barco.tags.InView, xml)[activeInput];
        }
    }, {
        barconame: 'PanZoomStorePix',
        barcolabel: 'Pan/Zoom',
        model: "Panz",
        ioType: 'input',
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.UncroppedOWIN
            }
        },
        selectXmlSpecific: function (xml) {
            "use strict";
            var activeInput = barco.utils.xmlselectnode(barco.tags.ActiveConn).textContent;
            return Ext.DomQuery.select(barco.tags.InView, xml)[activeInput];
        },
        inner: new Ext.XTemplate('<OWIN{tag}{outchannel}>{value}</OWIN{tag}{outchannel}>'),
        apply: function(obj, index) {
            "use strict";
            var template, tag, inv, innerObj, cmd;
            inv = this.selectXmlSpecific(barco.utils.store.rawxml);
            template = barco.tags.reverseTemplate(inv).template;
            
            tag = obj.tag;
            if(tag == 'Vpos') 
            {
                tag = 'VPos';
            }
            innerObj = {value:obj.value, tag:tag, outchannel:index};
        
            cmd = template.apply({value:this.inner.apply(innerObj)});
            return barco.utils.convertXmlToCmd(cmd);
        },
        //transforms from centered to top-left
        toViewValue: function(tag, value, index){
            "use strict";
            if(tag == "HPos" || tag == "Vpos") {
                var aoirect = barco.modelProxy.stores.AOIRectStore.data.items[index].data,
                    item = this.getAt(index),
                    sizeTag = tag == "HPos" ? "HSize" : "VSize",
                    t = Math.ceil((parseInt(item.data[sizeTag], 10) - parseInt(aoirect[sizeTag], 10)) / 2);
                return (value-parseInt(aoirect[tag], 10))+t;
            }
            return value;
        },
        fromViewValue: function(tag, value, index){
            "use strict";
            if(tag == "HPos" || tag == "Vpos") {
                var aoirect = barco.modelProxy.stores.AOIRectStore.data.items[index].data,
                    item = this.getAt(index),
                    sizeTag = tag == "HPos" ? "HSize" : "VSize",
                    t = Math.ceil((parseInt(item.data[sizeTag], 10) - parseInt(aoirect[sizeTag], 10)) / 2);
                return (value-t)+parseInt(aoirect[tag], 10);
            }
            return value;
        }
    }, {
        barconame: 'InputChanPipMaskStore',
        barcolabel: 'Input Channel PIP',
        model: barco.tags.PipMask,
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.PipMask
            }
        },
        selectXmlSpecific: barco.tags.getInputMgr,
        ioType: 'input'
    }, {
        barconame: 'SystemStore',
        barcolabel: 'System',
        model: barco.tags.SystemMgr,
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: barco.tags.SystemMgr
            }
        },
        opModeTempl: new Ext.XTemplate('<ImagePRO2 Num="0"><CmdType>1</CmdType><SystemMgr Num="0">'+
                                       '<OpModeCmd>{value}</OpModeCmd></SystemMgr></ImagePRO2>'),
        apply: function(obj, index)
        {
            "use strict";
            if(obj.tag==barco.tags.OpMode){
                 return this.opModeTempl.apply(obj);
            }
            return null;
        }
    }, {
        barconame: 'AOIRectStore',
        barcolabel: 'Area of Interest Rect',
        model: "Panz",
        proxy: {
            type: 'memory',
            reader: {
                type: 'xml',
                record: 'OutputChan ' + barco.tags.AOIRect
            }
        },
        ioType: 'output'
    }]
};

barco.TestPatternTypes = new Ext.data.Store({
    fields: ['value', 'text'],
    data: [{
        text: 'Off',
        value: 0
    }, {
        text: 'H Ramp',
        value: 1
    }, {
        text: 'V Ramp',
        value: 2
    }, {
        text: '100% Col Bars',
        value: 3
    }, {
        text: '16x16 Grid',
        value: 4
    }, {
        text: '32x32 Grid',
        value: 5
    }, {
        text: 'Burst',
        value: 6
    }, {
        text: '75% Col bars',
        value: 7
    }, {
        text: '50% Gray',
        value: 8
    }, {
        text: 'Gray Steps 1',
        value: 9
    }, {
        text: 'Gray Steps 2',
        value: 10
    }, {
        text: 'White',
        value: 11

    }, {
        text: 'Black',
        value: 12

    }, {
        text: 'SMPTE Bars',
        value: 13
    }]
});

barco.ColorSpaceTypes = new Ext.data.Store({
    fields: ['value', 'text'],
    data: [{
        text: 'RGB',
        value: 0
    }, {
        text: 'YCbCr',
        value: 1
    }/*, {
        text: 'Beta',
        value: 2
    }, {
        text: 'MII',
        value: 3
    }*/]
});

barco.AspectRatioTypes = new Ext.data.Store({
    fields: ['value', 'text'],
    data: [{
        text: 'Custom',
        value: 0
    },{
        text: '1:1',
        value: 1000
    }, {
        text: '3:2',
        value: 1500
    }, {
        text: '4:3',
        value: 1333
    }, {
        text: '5:4',
        value: 1250
    }, {
        text: '16:10',
        value: 1600
    }, {
        text: '16:9',
        value: 1778
    }/*,{
        text: 'Custom',
        value: 0
    }*/]
});

// Translate the value to the index to be used in the ICH command.
barco.GetAspectRatioCmdNum = function(value) {
    "use strict";
    switch (value) {
        case 0:
            return 6;
        case 1000:
            return 0;
        case 1500:
            return 1;
        case 1333:
            return 2;
        case 1250:
            return 3;
        case 1600:
            return 4;
        case 1778:
            return 5;
        default:
            barco.log("Invalid AspectRation Type found!!!");
            return 6;                
    }
};
        
Ext.regModel(barco.tags.VideoColor, {
    fields: [
        barco.tags.ContrR, 
        barco.tags.ContrG, 
        barco.tags.ContrB, 
        barco.tags.BrightR, 
        barco.tags.BrightG, 
        barco.tags.BrightB, 
        barco.tags.Hue, 
        barco.tags.Sat, 
        barco.tags.Contr, 
        barco.tags.Bright, 
        barco.tags.Gamma, 
        barco.tags.Mono, 
        barco.tags.ColorInvert, 
        {name: barco.tags.ColorSpace, type: 'int'},
        barco.tags.Sharp]
});

Ext.regModel(barco.tags.TestPattern, {
    fields: [
        {name: barco.tags.TestPatternMode, type: 'int'},
        barco.tags.RasterBoxMode,
        barco.tags.DiagMotion
    ]
});
/*
Ext.regModel(barco.tags.TestPatternMode, {
    fields: [
        {
            name: barco.tags.TestPatternMode, 
            mapping: barco.tags.TestPattern + ' ' + barco.tags.TestPatternMode,
            convert: function (obj, record) {
                "use strict";
                var mapobj = barco.TestPatternTypes.data.items[obj];
                if (mapobj !== undefined) {
                    //select field converts array to data store
                    obj = mapobj.data.text;
                } else {
                    console.log("Test pattern type not found: " + obj);
                }
                return obj;
            }
        }
    ]
});
 */
Ext.regModel("Mode3DOnly", {
    fields: [
        {name: barco.tags.Mode3D},
        {name: barco.tags.Swap3D}
    ]
});

Ext.regModel(barco.tags.IWIN, {
    fields: [        
        barco.tags.HSize,//, mapping: barco.tags.IWIN + " " + barco.tags.HSize},
        barco.tags.VSize,//, mapping: barco.tags.IWIN + " " + barco.tags.VSize},
        barco.tags.HPos,//, mapping: barco.tags.IWIN + " " + barco.tags.HPos},
        barco.tags.VPos//, mapping: barco.tags.IWIN + " " + barco.tags.VPos} ]
    ]
});

Ext.regModel(barco.tags.VideoFormat, {
    fields: [
        barco.tags.Name,
        barco.tags.HActive,
        barco.tags.VActive
    ]
});

Ext.regModel(barco.tags.InputChan, {
    fields: [        
        {name: barco.tags.AspectRatioType, type: 'int'},
        barco.tags.CustomAspectRatio
    ]
});

Ext.regModel(barco.tags.PipMask, {
    fields: [
        barco.tags.MaskTop,
        barco.tags.MaskBottom,
        barco.tags.MaskRight,
        barco.tags.MaskLeft
    ]
});

Ext.regModel(barco.tags.InView, {
    fields: [
        {name: barco.tags.PanZoomUnitValue}]
});

Ext.regModel("Panz", {
    fields: [
        {name: barco.tags.HSize},
        {name: barco.tags.VSize},
        {name: barco.tags.HPos}, //PanZoomPxl //UncroppedOWIN , mapping:'PanZoomPxl HPos'
        {name: barco.tags.VPos} ]
});

Ext.regModel(barco.tags.SystemMgr, {
    fields: [
        barco.tags.Lock,
        barco.tags.OpMode
    ]
});

Ext.regModel(barco.tags.InputMgr, {
    fields: [
        barco.tags.FreezeMode        
    ]
});