barco.InputMain = Ext.extend(Ext.form.FormPanel, {
    title: 'Input Main',
    id: 'Input Main',
    scroll: 'verticle',
    items:[{
        xtype: 'fieldset',
        defaults: {
            labelWidth: '35%',
            xtype: 'barcoslider',
            componentStore: "InputMainStore"
        },
    
        items: [
            barco.InputFormatSelector, 
            {
                xtype: 'selectfield',
                id: 'ColorSpace-select',
                noXMLWrite: false,
                name: barco.tags.ColorSpace,
                label: 'Colorspace',
                store: barco.ColorSpaceTypes
            }, 
            {
                name: 'Contr',
                label: 'Contrast',
                maxValue: 150,
                minValue: 25
            },{
                name: 'Bright',
                label: 'Brightness',
                maxValue: 150,
                minValue: 25
            },{
                name: 'Gamma',
                label: 'Gamma',
                maxValue: 3.3,
                minValue: 0.3,
                increment: 0.01
            }, 
            Ext.applyIf({id: 'input-select-3d-packing', componentStore: 'UInputStore'}, barco.mod3D.packingSelector),
            Ext.applyIf({id: 'input-select-3d-swap', componentStore: 'UInputMgrStore'}, barco.mod3D.swapSelector)
        ]
    }],
    DisableControlForQuadToDual: function() {
        "use strict";    
        var isQuadToDual, i, length;
        isQuadToDual = barco.utils.isQuadToDualMode();
        this.showWheelPlugin = !isQuadToDual;
            
        // main menu items
        length =  Ext.getCmp('Input Main').items.items[0].items.length;
        for (i=0; i < length; i+=1)
        {
            Ext.getCmp('Input Main').items.items[0].items.items[i].setDisabled(isQuadToDual);
            if(Ext.getCmp('Input Main').items.items[0].items.items[i].slidval){
                Ext.getCmp('Input Main').items.items[0].items.items[i].slidval.disabled = isQuadToDual;
            }
        }               
    },
    constructor: function(config) {
        "use strict";
        barco.modelProxy.listeners.addDefaultListeners(this);
        barco.InputMain.superclass.constructor.call(this, config);
        barco.modelProxy.enableGenericFormCallback(this, "InputMainStore");
        barco.modelProxy.enableGenericFormCallback(this, "InputFormatStore");       
        barco.modelProxy.enableGenericFormCallback(this, "UInputStore");
        
        this.x3DSwapSelect = Ext.getCmp('input-select-3d-swap');
        this.x3DPackingSelect = Ext.getCmp('input-select-3d-packing');
        this.x3DPackingSelect.on("change", function(){
            Ext.getCmp("theinputbar").lightInputBar();
        }, this);
        
        
        var visibleByInputCbk = function(xml){
            if(this.rendered && this.isVisible()) {
                this.items.items[0].setVisible(!barco.utils.isInputStatic());
                if(this.items.items[0].isVisible()){
                    this.x3DPackingSelect.setVisible(barco.utils.is3DMode());
                    this.x3DSwapSelect.setVisible(barco.utils.is3DMode());
                    if(barco.utils.is3DMode()){
                        this.x3DPackingSelect.setLROptions(true);
                    }
                }
                this.DisableControlForQuadToDual();
            }
        };
        barco.modelProxy.addCallback({func: visibleByInputCbk, scope: this, storename: 'SystemStore'});

        this.on('show', function() {
            var sel = Ext.getCmp("input-select");
            barco.setResolutionByIndex.call(sel, null, barco.modelProxy.stores[sel.componentStore]);
            visibleByInputCbk.call(this);
            this.DisableControlForQuadToDual();
        });
    }
});

barco.InputColorBalance = Ext.extend(barco.ColorBalance, {
    title: 'Color Balance',
    id: 'inputcolorbalance',
    hueMinMax:[-90,90],
    colorMinMax:[25,150],
    constructor: function(config) {
        "use strict";
        barco.InputColorBalance.superclass.constructor.call(this, config);
        barco.modelProxy.enableGenericFormCallback(this, "InputMainStore");
    
        var visibleByInputCbk = function(xml){
            if(this.rendered && this.isVisible()) {
                this.items.items[0].setVisible(!barco.utils.isInputStatic());
            }
        };
        this.on('show', visibleByInputCbk);
        barco.modelProxy.addCallback({
            func: visibleByInputCbk, 
            scope: this, 
            storename:'SystemStore' 
        });
    }
});

barco.InputAspectRatio = Ext.extend(Ext.form.FormPanel, {
    title: 'Input AspectRatio',
    id: 'Aspect Ratio',
    scroll: 'verticle',
    items: [{
        xtype: 'fieldset',
        title: 'Aspect Ratio',
        defaults: {         
            labelWidth: '35%',
            xtype: 'barcoslider',
            noXMLWrite: true,
            componentStore: "InputChanStore" 
        },
        items: [{
            xtype: 'selectfield',
            store: barco.AspectRatioTypes,
            name: barco.tags.AspectRatioType,
            id: barco.tags.AspectRatioType,
            label: 'Aspect Ratio Mode'
        }, {                            
            name: barco.tags.CustomAspectRatio,
            id: barco.tags.CustomAspectRatio,
            label: 'Custom Ratio',            
            maxValue: 10.0,
            minValue: 0.25,
            increment: 0.01            
        }]
    }
    ],
    index: 0,              
    changeOutputChannel: function(chan){
        "use strict";
        if(chan != 0 && chan != 1){
            return;
        }
        this.index = chan;
        barco.modelProxy.storeChanged('InputChanStore');   
        Ext.getCmp(barco.tags.AspectRatioType).valueBeforeChange = Ext.getCmp(barco.tags.AspectRatioType).getValue();
        Ext.getCmp(barco.tags.CustomAspectRatio).valueBeforeChange = Ext.getCmp(barco.tags.CustomAspectRatio).getValue();
    },    
    SendAspectRatioCmd: function(){
        "use strict";                
        var node = barco.utils.xmlselectnode(barco.tags.ActiveConn),
            currInput = parseInt(node.textContent, 10),
            channel = this.index;

        if (Ext.getCmp(barco.tags.AspectRatioType).valueBeforeChange != Ext.getCmp(barco.tags.AspectRatioType).getValue())
        {
            Ext.getCmp(barco.tags.AspectRatioType).valueBeforeChange = Ext.getCmp(barco.tags.AspectRatioType).getValue();
            if (Ext.getCmp(barco.tags.AspectRatioType).getValue() != 0) 
            {
                Ext.getCmp(barco.tags.CustomAspectRatio).valueBeforeChange = Ext.getCmp(barco.tags.AspectRatioType).getValue() / 1000.0;
                Ext.getCmp(barco.tags.CustomAspectRatio).setValue(Ext.getCmp(barco.tags.CustomAspectRatio).valueBeforeChange);
            }

            // DVI input requires specifying the input type. 
            if (currInput == 0) {            
                barco.utils.ascii('ich --con '+String(currInput)+' --ch ' + channel +
                    ' --itype '+ barco.utils.dviInputType() + 
                    ' --at ' + barco.GetAspectRatioCmdNum(Ext.getCmp(barco.tags.AspectRatioType).getValue()) + 
                    ' --ar ' + Ext.getCmp(barco.tags.AspectRatioType).getValue() / 1000.0);                                                            
            }           
            else {
                barco.utils.ascii('ich --con '+String(currInput)+' --ch ' + channel +
                    ' --at ' + barco.GetAspectRatioCmdNum(Ext.getCmp(barco.tags.AspectRatioType).getValue()) +
                    ' --ar ' + Ext.getCmp(barco.tags.AspectRatioType).getValue() / 1000.0);                                                            
            }	
        }

        if (Ext.getCmp(barco.tags.CustomAspectRatio).valueBeforeChange != Ext.getCmp(barco.tags.CustomAspectRatio).getValue())
        {
            Ext.getCmp(barco.tags.CustomAspectRatio).valueBeforeChange = Ext.getCmp(barco.tags.CustomAspectRatio).getValue();

            // DVI input requires specifying the input type. 
            if (currInput == 0) {            
                barco.utils.ascii('ich --con '+String(currInput)+' --ch ' + channel +
                    ' --itype '+ barco.utils.dviInputType() + 
                    ' --ar ' + Ext.getCmp(barco.tags.CustomAspectRatio).getValue());                                   
            }
            else {
                barco.utils.ascii('ich --con '+String(currInput)+' --ch ' + channel +                                
                    ' --ar ' + Ext.getCmp(barco.tags.CustomAspectRatio).getValue());                                              
            }	                        
        }                                                       
    },
    DisableControlForQuadToDual: function() {
        "use strict";
        if(!this.rendered){
            return;
        }
        var isQuadToDual, i, length;
        
        isQuadToDual = barco.utils.isQuadToDualMode();
        this.showWheelPlugin = !isQuadToDual;
        // aspect ratio menu options
        length =  Ext.getCmp('Aspect Ratio').items.items[1].items.length;
        for (i=0; i < length; i+=1)
        {
            Ext.getCmp('Aspect Ratio').items.items[1].items.items[i].setDisabled(isQuadToDual);
            Ext.getCmp('Aspect Ratio').items.items[1].items.items[i].showWheelPlugin = !isQuadToDual;
            if(Ext.getCmp('Aspect Ratio').items.items[1].items.items[i].slidval){
                Ext.getCmp('Aspect Ratio').items.items[1].items.items[i].slidval.disabled = isQuadToDual;
            }
        } 
    },
    constructor: function(config) {
        "use strict";
        var hideShowCustomRatioWithStore, hideShowCustomRatio, onAspectRatioChange, onInputChange; 
        
        barco.modelProxy.listeners.addDefaultListeners(this);
        barco.InputAspectRatio.superclass.constructor.call(this, config);
        barco.modelProxy.enableGenericFormCallback(this, 'InputChanStore');                

        onInputChange = function(){
            if(this.rendered && this.isVisible()) {
                this.items.items[1].setVisible(!barco.utils.isInputStatic());
            }
        };
       
        hideShowCustomRatioWithStore = function(xml, store){
            if(!this.rendered || barco.modelProxy.stores.InputChanStore.data.items.length == 0)
            {
                return;
            }
            //console.log("pushing data " + store.barconame);
            this.barcostore = store;
                
            if (store.data.items[this.index].data.PipAR == 0) {                                    
                Ext.getCmp(barco.tags.CustomAspectRatio).setVisible(true);
            }
            else {
                Ext.getCmp(barco.tags.CustomAspectRatio).setVisible(false);
            }
        };
        
        hideShowCustomRatio = function(){                
            if (Ext.getCmp(barco.tags.AspectRatioType).value == 0) {                                   
                Ext.getCmp(barco.tags.CustomAspectRatio).setVisible(true);
            }
            else {
                Ext.getCmp(barco.tags.CustomAspectRatio).setVisible(false);            
            }
        };
        
        onAspectRatioChange = function(){
            hideShowCustomRatio();
            this.SendAspectRatioCmd();
        };        

        barco.modelProxy.addCallback({
            func: hideShowCustomRatioWithStore, 
            scope: this, 
            storename:'InputChanStore'
        });
        
        barco.modelProxy.addCallback({
            func: onInputChange, 
            scope: this, 
            storename:'InputChanStore'
        });
        
        barco.modelProxy.addCallback({
            func: this.DisableControlForQuadToDual, 
            scope: this
        });
        
        this.on('show', onInputChange);
        
        barco.evtManager.on("ChannelChange", this.changeOutputChannel, this);                             
                
        this.on('afterrender',hideShowCustomRatio);
        this.on('show',this.DisableControlForQuadToDual);  
        
        Ext.getCmp(barco.tags.AspectRatioType).on("change", onAspectRatioChange, this);                    
        Ext.getCmp(barco.tags.CustomAspectRatio).on("change", onAspectRatioChange, this);                    
    }
});

barco.InputSizingAdjust = Ext.extend(Ext.form.FormPanel, {
    title: 'Sizing Adjust',
    id: 'Sizing Adjust',
    scroll: 'verticle',
    plugins: [{
        ptype: 'wheelplugin'
    }],
    items: [{
        xtype: 'fieldset',
        title: 'Input Window',
        defaults: {
            labelWidth: '35%',
            xtype: 'barcoadjslider',
            componentStore: 'InputChanIWINStore',
            minmaxTag: barco.tags.IWIN,
            cls: 'needwheel',
            showWheelPlugin: true
        },
        items: [{
                id: 'saHsize',
                name: barco.tags.HSize,
                label: 'H Size',
                maxValue: barco.tags.constraints[barco.tags.HSize],
                activeTag: {store:'InputFormatStore', tag:'HActive'}
            },{
                id: 'saVsize',
                name: barco.tags.VSize,
                label: 'V Size',
                maxValue: barco.tags.constraints[barco.tags.VSize],
                activeTag: {store:'InputFormatStore', tag:'VActive'}
            },{
                id: 'saHpos',
                name: barco.tags.HPos,
                label: 'H Pos',
                maxValue: barco.tags.constraints[barco.tags.HSize],
                activeTag: {store:'InputFormatStore', tag:'HActive'}
            },{
                id: 'saVpos',
                name: barco.tags.VPos,
                label: 'V Pos',
                maxValue: barco.tags.constraints[barco.tags.HSize],
                activeTag: {store:'InputFormatStore', tag:'VActive'}
            }]
        }, {
        xtype: 'fieldset',
        title: 'Mask',
        defaults: {
          labelWidth: '35%',
          xtype: 'barcoslider',
          componentStore: 'InputChanPipMaskStore'
        },
        items: [{
                id: 'saMaskTop',
                name: barco.tags.MaskTop,
                label: 'Mask Top %',
                oppositeTag: 'saMaskBottom'
            },{
                id: 'saMaskBottom',
                name: barco.tags.MaskBottom,
                label: 'Mask Bottom %',
                oppositeTag: 'saMaskTop'
            },{
                id: 'saMaskLeft',
                name: barco.tags.MaskLeft,
                label: 'Mask Left %',
                oppositeTag: 'saMaskRight'
            },{
                id: 'saMaskRight',
                name: barco.tags.MaskRight,
                label: 'Mask Right %',
                oppositeTag: 'saMaskLeft'
            }
        ]
    }],
    index: 0,
    changeOutputChannel: function(chan){
        "use strict";
        if(chan != 0 && chan != 1){
            return;
        }
        this.index = chan;       
        barco.modelProxy.storeChanged('InputChanIWINStore');        
        barco.modelProxy.storeChanged('InputChanPipMaskStore');
    },    
    DisableControlForMinDelay: function() {
        "use strict";    
        var isMinDelay, i, length;
        
        isMinDelay = barco.utils.isMinDelayMode();        
        this.showWheelPlugin = !isMinDelay;
            
        // sizing menu items
        length =  Ext.getCmp('Sizing Adjust').items.items[1].items.length;
        for (i=0; i < length; i+=1)
        {
            Ext.getCmp('Sizing Adjust').items.items[1].items.items[i].setDisabled(isMinDelay);                                    
            Ext.getCmp('Sizing Adjust').items.items[1].items.items[i].slidval.disabled = isMinDelay;                    
            Ext.getCmp('Sizing Adjust').items.items[1].items.items[i].showWheelPlugin = !isMinDelay;
        }  
        
        // masking menu items
        length =  Ext.getCmp('Sizing Adjust').items.items[2].items.length;
        for (i=0; i < length; i+=1)
        {
            Ext.getCmp('Sizing Adjust').items.items[2].items.items[i].setDisabled(isMinDelay);                                    
            Ext.getCmp('Sizing Adjust').items.items[2].items.items[i].slidval.disabled = isMinDelay;                    
        }

        if(!barco.evtManager.ShowChannelModeButton){
            barco.evtManager.fireEvent('ShowChannelModeButton', false);//test this by starting mode std -> quad -> std (options should be back)
        }
    },
    DisableControlForQuadToDual: function() {
        "use strict";    
        if(!this.rendered){
            return;
        }
        var isQuadToDual, i, length;
        
        isQuadToDual = barco.utils.isQuadToDualMode();
        this.showWheelPlugin = !isQuadToDual;
        
        // sizing menu items
        length =  Ext.getCmp('Sizing Adjust').items.items[1].items.length;
        for (i=0; i < length; i+=1)
        {
            Ext.getCmp('Sizing Adjust').items.items[1].items.items[i].setDisabled(isQuadToDual);                                    
            Ext.getCmp('Sizing Adjust').items.items[1].items.items[i].slidval.disabled = isQuadToDual;                    
            Ext.getCmp('Sizing Adjust').items.items[1].items.items[i].showWheelPlugin = !isQuadToDual;
        }  
        
        // masking menu items
        length =  Ext.getCmp('Sizing Adjust').items.items[2].items.length;
        for (i=0; i < length; i+=1)
        {
            Ext.getCmp('Sizing Adjust').items.items[2].items.items[i].setDisabled(isQuadToDual);                                    
            Ext.getCmp('Sizing Adjust').items.items[2].items.items[i].slidval.disabled = isQuadToDual;                    
        }               
    },
    setOldValues: function(){
        "use strict";
        var x, cmp, cmps = ["saHsize", "saVsize", "saHpos", "saVpos", 
                "saMaskTop", "saMaskBottom", "saMaskLeft", "saMaskRight"];
        for(x=0; x<cmps.length; x+=1)
        {
            cmp = Ext.getCmp(cmps[x]);
            cmp.oldValueBarco = cmp.getValue();
        }
    },
    constructor: function(config) {
        "use strict";
        barco.modelProxy.listeners.addDefaultListeners(this);
        barco.InputSizingAdjust.superclass.constructor.call(this, config);        
        barco.modelProxy.enableGenericFormCallback(this, 'InputChanPipMaskStore');
        
        //InputSizingAdjust requeries, so don't automatically update local xml
        this.dontUpdateLocal = true;
        
        var inputChanStoreCbk = function(xml, store){
                var winh, winv, centered, data, hoffset, voffset, cmp, cmps, x;
                if(!this.rendered || barco.modelProxy.stores.InputFormatStore.data.items.length == 0)
                {
                    return;
                }
                //console.log("pushing data " + store.barconame);
                this.barcostore = store;
                
                winh = parseInt(barco.modelProxy.stores.InputFormatStore.data.items[0].data.HActive, 10);
                winv = parseInt(barco.modelProxy.stores.InputFormatStore.data.items[0].data.VActive, 10);
                
                centered = barco.utils.topLeftToCenter(store.data.items[this.index].data, winh, winv);
                
                data = {};
                data.HPos = centered[0];
                data.Vpos = centered[1];
                data.HSize = store.data.items[this.index].data.HSize;
                data.VSize = store.data.items[this.index].data.VSize;
                
                hoffset = Math.floor((data.HSize-winh) / 2);
                voffset = Math.floor((data.VSize-winv) / 2);
                
                Ext.getCmp("saHsize").adjustMinMax(0);
                Ext.getCmp("saVsize").adjustMinMax(0);
                Ext.getCmp("saHpos").adjustMinMax(hoffset);
                Ext.getCmp("saVpos").adjustMinMax(voffset, true);
                
                this.changesLocked = true;
                this.loadModel({data:data});
                this.changesLocked = false;
                
                if(!this.dragging){
                   this.setOldValues();
                }
            }, x, cmp, visibleByInputCbk,
            cmps = ["saHsize", "saVsize", "saHpos", "saVpos", 
                    "saMaskTop", "saMaskBottom", "saMaskLeft", "saMaskRight"],
            requeryFun = function() {
                var ind =  Ext.getCmp('Sizing Adjust').index,
                    node = barco.modelProxy.stores.InputChanIWINStore.data.items[ind].raw;
                barco.utils.refreshNode(node, 'InputChanIWINStore');
            },
            requeryBothFun = function() { //on show
                this.index = barco.evtManager.ChannelChange;
                barco.utils.refreshNode(barco.modelProxy.stores.InputChanIWINStore.data.items[0].raw, 'InputChanIWINStore');
                barco.utils.refreshNode(barco.modelProxy.stores.InputChanIWINStore.data.items[1].raw, 'InputChanIWINStore');
            },
            maskAdjustFun = function() {
                var opp = Ext.getCmp(this.oppositeTag);
                if(opp.rendered){
                    opp.setMax(100-this.getValue());
                }
            };
            
        barco.modelProxy.addCallback({
            func: inputChanStoreCbk, 
            scope: this, 
            storename:'InputChanIWINStore'
        });
        
        barco.modelProxy.addCallback({
            func: this.DisableControlForQuadToDual, 
            scope: this
        });
            
        for(x=0; x<cmps.length; x+=1)
        {
            cmp = Ext.getCmp(cmps[x]);
            cmp.ajaxChangeSuccess = requeryFun;
            //for mask
            if(x > 3)
            {
                cmp.on('change', maskAdjustFun, cmp);
            }
        }

        this.on('beforeshow', requeryBothFun);
        barco.evtManager.on("ChannelChange", this.changeOutputChannel, this);
        
        //SystemStore
        visibleByInputCbk = function(xml){
            if(this.rendered && this.isVisible())
            {
                this.items.items[1].setVisible(!barco.utils.isInputStatic());
                this.items.items[2].setVisible(!barco.utils.isInputStatic());
            }
        };
        barco.modelProxy.addCallback({
            func: visibleByInputCbk,
            scope: this,
            storename:'SystemStore'
        });

        this.on('show', function(){
            visibleByInputCbk.call(this); 
            this.DisableControlForMinDelay();
            this.DisableControlForQuadToDual();
        });
    }
});


barco.InputConfig = Ext.extend(Ext.form.FormPanel, {
    title: 'Input Config',
    id: 'inputconfig',
    scroll: 'verticle',
    items:[{
        xtype: 'fieldset',
        defaults: {
            labelWidth: '45%',
            xtype: 'barcoslider',
            componentStore: "InputMainStore"
        },
    
        items: [
            {
                xtype: 'barcobuttonfield',
                buttonlabel: 'Save Config',
                disableButtonOnEmpty: true,
                placeHolder: 'filename',
                noTextLabel: true,
                value: '',
                id: 'saveConfigField',
                listeners: {
                    buttonclick: function(item) {
                        "use strict";
                        item.disable();
                        barco.utils.ascii({
                            request:'icsav '+barco.utils.selectedInput()+' '+item.textfield.dom.value,
                            success: function() {
                                Ext.getCmp("inputconfig").showMsg('Config saved', 'msgBlack');
                                item.enable();
                            },
                            failure: function() {
                                Ext.getCmp("inputconfig").showMsg('Save config failed', 'msgRed');
                                item.enable();
                            }
                        });
                    }
                }
            },
            {
                xtype: 'barcobuttonfield',
                buttonlabel: 'Reset Config',
                placeHolder: 'text goes here',
                noTextLabel: true,
                useMask: true,
                name: 'akak',
                listeners: {
                    buttonclick: function(item) {
                        "use strict";
                        barco.utils.ascii({
                            request:'icrst '+barco.utils.selectedInput(),
                            success: function() {
                                Ext.getCmp("inputconfig").showMsg('Config reseted', 'msgBlack');
                                barco.utils.refreshNode(barco.tags.getInputMgr(barco.utils.store.rawxml), undefined, 
                                    function(){
                                        item.enable();
                                    });
                            },
                            failure: function() {
                                Ext.getCmp("inputconfig").showMsg('Config reset failed', 'msgRed');
                                item.enable();
                            }
                        });
                    }
                }
            }]
    },
    {
        id: 'configMsg',
        xtype:'barcomodelessmessage',
        text:'Save config failed.',
        hidden: true
    }],
    constructor: function(config) {
        "use strict";
        barco.InputConfig.superclass.constructor.call(this, config);
        this.configMsg = Ext.getCmp("configMsg");
        this.filenames = {1:'',2:'',3:'',4:'',5:'',6:''};
        this.lastInput = 1;
        var onInputChange = function(){
            var visib = barco.utils.isInputStatic(), saveConfigField = Ext.getCmp("saveConfigField");
            this.items.items[0].setVisible(!visib);
            if(!visib)
            {
                this.filenames[this.lastInput] = saveConfigField.getValue();
                this.lastInput = barco.utils.selectedInput();
                saveConfigField.setValue(this.filenames[this.lastInput]);
            }
        };
        barco.modelProxy.addCallback({
            func: onInputChange, 
            scope: this, 
            storename:'SystemStore'
        });
        this.on('show', onInputChange);
    },
    showMsg: function(text, colorCls) {
        "use strict";
        this.configMsg.setText(text, colorCls);
        this.configMsg.popInAndOut();
    },
    submit: function() {
        "use strict";
        var saveConfigField = Ext.getCmp('saveConfigField');
        saveConfigField.fireEvent('buttonclick', saveConfigField);

    }
});


