
barco.OutputMain = Ext.extend(Ext.form.FormPanel, {
    title: 'Output Main',
    id: 'Output Main',
    scroll: 'verticle',
    items: [{
        xtype: 'fieldset',
        title: 'Output Format',
        defaults: {
            labelWidth: '35%'
        },
        items: [
                Ext.applyIf({id: 'output-1-select', label: 'Channel 1', index: 0}, barco.OutputFormatSelector),
                Ext.applyIf({id: 'output-2-select', label: 'Channel 2', index: 1}, barco.OutputFormatSelector),
                Ext.applyIf({id: 'output-select-3d-packing', componentStore: 'OutputMgrStore'}, barco.mod3D.packingSelector),
                Ext.applyIf({id: 'output-select-3d-swap', componentStore: 'OutputMgrStore'}, barco.mod3D.swapSelector)
            ]
        }],
    hideAndShow: function(){
        "use strict";
        Ext.getCmp('output-2-select').setVisible(barco.utils.isDualMode());
        Ext.getCmp('output-select-3d-packing').setVisible(barco.utils.is3DMode());
        Ext.getCmp('output-select-3d-swap').setVisible(barco.utils.is3DMode());
        this.x3DPackingSelect = Ext.getCmp('output-select-3d-packing');
        if(barco.utils.is3DMode()){
            this.x3DPackingSelect.setLROptions(false);
        }          
    },
    DisableControlForMinDelayQuadToDual: function() {
        "use strict";    
        Ext.getCmp('output-1-select').setDisabled(barco.utils.isMinDelayMode() || barco.utils.isQuadToDualMode());         
    },
    constructor: function(config) {
        "use strict";
        barco.modelProxy.listeners.addDefaultListeners(this);
        barco.OutputMain.superclass.constructor.call(this, config);
        barco.modelProxy.enableGenericFormCallback(this, "OutputMgrStore");
        
        barco.modelProxy.addCallback({func: this.hideAndShow, scope: this, storename:'SystemStore'});        
        this.on('show', function() {
            var sels = ["output-1-select", "output-2-select"], sel, outch;
            for(outch=0; outch<sels.length; outch+=1){
                sel = Ext.getCmp(sels[outch]);
                barco.setResolutionByIndex.call(sel, null, barco.modelProxy.stores[sel.componentStore]);
            }            
            this.hideAndShow();
            this.DisableControlForMinDelayQuadToDual();
        });       
    }
});

barco.OutputEffects = {
    changeBoth: function(newval){
        "use strict";
        if(barco.evtManager.ChannelBoth) {
            this.noafterchange = true;//avoid recursion
            this.ownerCt.ownerCt.index = (this.ownerCt.ownerCt.index +1) %2;
            barco.modelProxy.listeners.fieldset.change(this, null, newval);
            this.ownerCt.ownerCt.index = (this.ownerCt.ownerCt.index +1) %2;
            this.noafterchange = false;
        }
    },
    dragBoth: function(newval){
        "use strict";
        if(barco.evtManager.ChannelBoth) {
            var otherIndex = (this.ownerCt.ownerCt.index +1) %2,
                xml = barco.modelProxy.stores.OutputEffectsStore.apply({
                    tag: this.name, 
                    value: newval
                }, otherIndex);
            barco.utils.ajaxRequest({request: xml});
        }
    },
    addOrRemoveAsterisk: function(cmp){
        "use strict";
        if(cmp.getValue() != barco.modelProxy.stores.OutputEffectsStore.getAt(1).data[cmp.name]){
            barco.utils.asteriskIcon().add(cmp);
        }
        else{
            barco.utils.asteriskIcon().remove(cmp);
        }
    },   
    populatePanel: function(){
        "use strict";       
        if(!this.rendered || !this.isVisible() ){
            return;
        }        
        var fsCmp = this.getComponent(1), ic=0, cmp;
        if(barco.evtManager.ChannelBoth){
            this.index = 0;
            barco.modelProxy.genericCallBack.call(this, null, barco.modelProxy.stores.OutputEffectsStore);
            while((cmp = fsCmp.getComponent(ic))!==undefined){
                barco.OutputEffects.addOrRemoveAsterisk(cmp);
                ic += 1;
            }
        }
        else{
            while((cmp = fsCmp.getComponent(ic))!==undefined){
                barco.utils.asteriskIcon().remove(cmp);
                ic += 1;
            }
            barco.modelProxy.genericCallBack.call(this, null, barco.modelProxy.stores.OutputEffectsStore);
        }        
    }
};

barco.OutputColorBalance = Ext.extend(barco.ColorBalance, {
    title: 'Color Balance',
    id: 'outputcolorbalance',
    hueMinMax:[-180,180],
    colorMinMax:[0,200],
    index: 0,
    changeOutputChannel: function(chan){
        "use strict";
        if(chan != 0 && chan != 1){
            return;
        }
        this.index = chan;
        barco.modelProxy.genericCallBack.call(this, barco.utils.store.rawxml, barco.modelProxy.stores.OutputEffectsStore);
    },
    DisableControlForMinDelayQuadToDual: function() {
        "use strict";
        if(!this.rendered){
            return;
        }
        var isMinDelay, isQuadToDual, i, length; 
        length = Ext.getCmp('outputcolorbalance').fields.length;
        isMinDelay = barco.utils.isMinDelayMode();
        isQuadToDual = barco.utils.isQuadToDualMode();
        for (i=0; i < length; i+=1)
        {
            Ext.getCmp('outputcolorbalance').fields.items[i].setDisabled(isMinDelay || isQuadToDual);                                    
            Ext.getCmp('outputcolorbalance').fields.items[i].slidval.disabled = (isMinDelay || isQuadToDual);                    
        }               
    },
    itemDefaults: {
        afterchange: barco.OutputEffects.changeBoth,
        afterdrag: barco.OutputEffects.dragBoth
    },
    constructor: function(config) {
        "use strict";
        barco.OutputColorBalance.superclass.constructor.call(this, config);
        
        barco.modelProxy.addCallback({
            func: barco.OutputEffects.populatePanel,
            scope: this, 
            storename: 'OutputEffectsStore'
        });
        barco.modelProxy.addCallback({
            func: this.DisableControlForMinDelayQuadToDual, 
            scope: this
        });
        this.on('show', function(){                        
            barco.OutputEffects.populatePanel.call(this);
            this.DisableControlForMinDelayQuadToDual();
        });        
        
        this.index = barco.evtManager.ChannelChange;
        barco.evtManager.on("ChannelChange", this.changeOutputChannel, this);
        barco.evtManager.on("ChannelBoth", barco.OutputEffects.populatePanel, this);
        
        this.on("afterrender", function(){
            var fsCmp = this.getComponent(1), ic=0, cmp, parent=this,
                cbk = function(){
                    if(!parent.changesLocked && barco.evtManager.ChannelBoth){
                        barco.utils.asteriskIcon().remove(this);
                    }
                };
            while((cmp = fsCmp.getComponent(ic))!==undefined){
                cmp.on("change", cbk, cmp);
                ic += 1;
            }
        }, this);
    }
});

barco.OutputColorEffects = Ext.extend(Ext.form.FormPanel, {
    title: 'Color Effects',
    id: 'Color Effects',
    scroll: 'verticle',
    items: [{
        xtype: 'fieldset',
        title: 'Color Effects',
        defaults: {
           labelWidth: '35%',
           xtype: 'barcoslider',
           afterchange: barco.OutputEffects.changeBoth,
           afterdrag: barco.OutputEffects.dragBoth
        },
        items: [{
                name: 'Contr',
                label: 'Contrast',
                maxValue: 200
            },{
                name: 'Bright',
                label: 'Brightness',
                maxValue: 200
            },{
                name: 'Gamma',
                label: 'Gamma',
                increment: 0.01,
                minValue: 0.3,
                maxValue: 3.3
            },{
                xtype: 'togglefield',
                name: 'Mono',
                label: 'Monochrome'
            },{
                xtype: 'togglefield',
                name: 'ColorInvert',
                label: 'Invert'
            },{
                name: 'Sharp',
                label: 'Sharpness',
                minValue: -10,
                maxValue: 10
            }
        ]
    }],
    index: 0,
    changeOutputChannel: function(chan){
        "use strict";
        if(chan != 0 && chan != 1){
            return;
        }
        this.index = chan;
        barco.modelProxy.genericCallBack.call(this, barco.utils.store.rawxml, barco.modelProxy.stores.OutputEffectsStore);
    },
    DisableControlForMinDelayQuadToDual: function() {
        "use strict";
        if(!this.rendered){
            return;
        }
        var isMinDelay, isQuadToDual, i, length; 
        length = Ext.getCmp('Color Effects').fields.length;
        isMinDelay = barco.utils.isMinDelayMode();
        isQuadToDual = barco.utils.isQuadToDualMode();
        for (i=0; i < length; i+=1)
        {
            Ext.getCmp('Color Effects').fields.items[i].setDisabled(isMinDelay || isQuadToDual);           
            if (Ext.getCmp('Color Effects').fields.items[i].xtype == "barcoslider")
            {
                Ext.getCmp('Color Effects').fields.items[i].slidval.disabled = (isMinDelay || isQuadToDual);        
            }
        }               
    },
    constructor: function(config) {
        "use strict";
        barco.modelProxy.listeners.addDefaultListeners(this);
        barco.OutputColorEffects.superclass.constructor.call(this, config);
        
        barco.modelProxy.addCallback({
            func: barco.OutputEffects.populatePanel,
            scope: this, 
            storename: 'OutputEffectsStore'
        });
        barco.modelProxy.addCallback({
            func: this.DisableControlForMinDelayQuadToDual, 
            scope: this
        });
        this.on('show', function() {
            barco.OutputEffects.populatePanel.call(this);
            this.DisableControlForMinDelayQuadToDual();
        }, this);
        
        this.index = barco.evtManager.ChannelChange;
        barco.evtManager.on("ChannelChange", this.changeOutputChannel, this);
        barco.evtManager.on("ChannelBoth", barco.OutputEffects.populatePanel, this);
        
        this.on("afterrender", function(){
            var fsCmp = this.getComponent(1), ic=0, cmp, parent=this,
                cbk = function(){
                    if(!parent.changesLocked && barco.evtManager.ChannelBoth){
                        barco.utils.asteriskIcon().remove(this);
                    }
                };
            while((cmp = fsCmp.getComponent(ic))!==undefined){
                cmp.on("change", cbk, cmp);
                ic += 1;
            }            
        }, this);
    }
});

barco.OutputAOIRect = Ext.extend(Ext.form.FormPanel, {
    title: 'Area of Interest',
    id: 'outputaoirect',
    scroll: 'verticle',
    plugins: [{
        ptype: 'wheelplugin'
    }],
    items: [{
        xtype: 'fieldset',
        title: 'Area of Interest',
        defaults: {
            labelWidth: '35%',
            xtype: 'barcoadjslider',
            componentStore: 'AOIRectStore',
            cls: 'needwheel',
            showWheelPlugin: true
        },
        items: [{
                id: 'aoiHsize',
                name: barco.tags.HSize,
                label: 'H Size'
            },{
                id: 'aoiVsize',
                name: barco.tags.VSize,
                label: 'V Size'
            },{
                id: 'aoiHpos',
                name: barco.tags.HPos,
                label: 'H Pos'
            },{
                id: 'aoiVpos',
                name: barco.tags.VPos,
                label: 'V Pos'
            }
        ]
    }],
    index: 0, //output channel
    changeOutputChannel: function(chan){
        "use strict";
        if(chan != 0 && chan != 1){
            return;
        }
        this.index = chan;
        this.populatePanel();
    },
    populatePanel: function(rawxml, store)
    {
        "use strict";
        if(!this.rendered || !this.isVisible()){
            return;
        }
        Ext.getCmp("aoiHsize").adjustMinMax();
        Ext.getCmp("aoiVsize").adjustMinMax();
        Ext.getCmp("aoiHpos").adjustMinMax();
        Ext.getCmp("aoiVpos").adjustMinMax();
        
        this.changesLocked = true;
        if(store){
            this.barcostore = store;
        }
        this.loadModel(this.barcostore.data.items[this.index]);
        this.changesLocked = false;
        
        if(!this.dragging){
            var cmps = ["aoiHsize", "aoiVsize", "aoiHpos", "aoiVpos"], icmp, x;
            for(x=0; x<cmps.length; x+=1)
            {
                icmp = Ext.getCmp(cmps[x]);
                icmp.oldValueBarco = icmp.getValue();
            }
        }
    },
    DisableControlForMinDelayQuadToDual: function() {
        "use strict";
        if(!this.rendered){
            return;
        }
        var isMinDelay, isQuadToDual, i, length; 
        length = Ext.getCmp('outputaoirect').fields.length;
        isQuadToDual = barco.utils.isQuadToDualMode();
        isMinDelay = barco.utils.isMinDelayMode();
        this.showWheelPlugin = (!isMinDelay && !isQuadToDual);
        for (i=0; i < length; i+=1)
        {
            Ext.getCmp('outputaoirect').fields.items[i].setDisabled(isMinDelay || isQuadToDual);           
            Ext.getCmp('outputaoirect').fields.items[i].slidval.disabled = (isMinDelay || isQuadToDual);     
            Ext.getCmp('outputaoirect').fields.items[i].showWheelPlugin = (!isMinDelay && !isQuadToDual);
        }               
    },
    constructor: function(config) {
        "use strict";
        barco.modelProxy.listeners.addDefaultListeners(this);
        barco.OutputAOIRect.superclass.constructor.call(this, config);
        
        barco.modelProxy.addCallback({
            func: this.populatePanel, 
            scope: this, 
            storename:'AOIRectStore'
        });
        barco.modelProxy.addCallback({
            func: this.DisableControlForMinDelayQuadToDual, 
            scope: this
        });
        var x, cmp, me=this,
            cmps = ["aoiHsize", "aoiVsize", "aoiHpos", "aoiVpos"],
        requeryFun = function() {
            var node = barco.modelProxy.stores.AOIRectStore.data.items[me.index].raw, icmp;
            barco.utils.refreshNode(node, 'AOIRectStore');
        };
        for(x=0; x<cmps.length; x+=1)
        {
            cmp = Ext.getCmp(cmps[x]);
            cmp.ajaxChangeSuccess = requeryFun;
        }
        barco.evtManager.on("ChannelChange", this.changeOutputChannel, this);
        this.on("show", function(){
            this.index = barco.evtManager.ChannelChange;
            barco.utils.refreshNode(barco.modelProxy.stores.AOIRectStore.data.items[0].raw, 'AOIRectStore');
            barco.utils.refreshNode(barco.modelProxy.stores.AOIRectStore.data.items[1].raw, 'AOIRectStore');
            this.DisableControlForMinDelayQuadToDual();
        });
    }
});
